package reborncore.jsonDestroyers.block;

import com.google.common.collect.Lists;
import net.minecraft.client.renderer.block.model.BakedQuad;
import net.minecraft.client.renderer.block.model.BreakingFour;
import net.minecraft.client.renderer.block.model.ItemCameraTransforms;
import net.minecraft.client.renderer.block.model.ModelBlock;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.resources.model.IBakedModel;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.HashMap;
import java.util.List;

public class MultiTexturedBasedModel implements IBakedModel {
    protected final List<BakedQuad> generalQuads;
    protected final List<List<BakedQuad>> faceQuads;
    protected final boolean ambientOcclusion;
    protected final boolean gui3d;
    protected final HashMap<EnumFacing, TextureAtlasSprite> texture;
    protected final ItemCameraTransforms cameraTransforms;

    public MultiTexturedBasedModel(List<BakedQuad> generalQuads, List<List<BakedQuad>> faceQuads, boolean ambientOcclusion, boolean gui3d, HashMap<EnumFacing, TextureAtlasSprite> texture, ItemCameraTransforms cameraTransforms) {
        this.generalQuads = generalQuads;
        this.faceQuads = faceQuads;
        this.ambientOcclusion = ambientOcclusion;
        this.gui3d = gui3d;
        this.texture = texture;
        this.cameraTransforms = cameraTransforms;
    }

    public List<BakedQuad> func_177551_a(EnumFacing side) {
        return (List) this.faceQuads.get(side.ordinal());
    }

    public List<BakedQuad> func_177550_a() {
        return this.generalQuads;
    }

    public boolean func_177555_b() {
        return this.ambientOcclusion;
    }

    public boolean func_177556_c() {
        return this.gui3d;
    }

    public boolean func_177553_d() {
        return false;
    }

    public TextureAtlasSprite func_177554_e() {
        return texture.get(EnumFacing.DOWN);
    }

    public ItemCameraTransforms func_177552_f() {
        return this.cameraTransforms;
    }

    @SideOnly(Side.CLIENT)
    public static class Builder {
        private final List<BakedQuad> builderGeneralQuads;
        private final List<List<BakedQuad>> builderFaceQuads;
        private final boolean builderAmbientOcclusion;
        private HashMap<EnumFacing, TextureAtlasSprite> builderTexture;
        private boolean builderGui3d;
        private ItemCameraTransforms builderCameraTransforms;

        public Builder(ModelBlock modelBlock) {
            this(modelBlock.func_178309_b(), modelBlock.func_178311_c(), modelBlock.func_181682_g());
        }

        public Builder(IBakedModel iBakedModel, HashMap<EnumFacing, TextureAtlasSprite> builderTexture) {
            this(iBakedModel.func_177555_b(), iBakedModel.func_177556_c(), iBakedModel.func_177552_f());
            this.builderTexture = builderTexture;

            for (EnumFacing enumfacing : EnumFacing.values()) {
                this.addFaceBreakingFours(iBakedModel, builderTexture.get(enumfacing), enumfacing);
            }

            this.addGeneralBreakingFours(iBakedModel, builderTexture.get(EnumFacing.DOWN));
        }

        private void addFaceBreakingFours(IBakedModel iBakedModel, TextureAtlasSprite textureAtlasSprite, EnumFacing facing) {
            for (BakedQuad bakedquad : iBakedModel.func_177551_a(facing)) {
                this.addFaceQuad(facing, new BreakingFour(bakedquad, textureAtlasSprite));
            }
        }

        private void addGeneralBreakingFours(IBakedModel iBakedModel, TextureAtlasSprite textureAtlasSprite) {
            for (BakedQuad bakedquad : iBakedModel.func_177550_a()) {
                this.addGeneralQuad(new BreakingFour(bakedquad, textureAtlasSprite));
            }
        }

        private Builder(boolean builderAmbientOcclusion, boolean builderGui3d, ItemCameraTransforms builderCameraTransforms) {
            this.builderGeneralQuads = Lists.<BakedQuad>newArrayList();
            this.builderFaceQuads = Lists.<List<BakedQuad>>newArrayListWithCapacity(6);

            for (EnumFacing enumfacing : EnumFacing.values()) {
                this.builderFaceQuads.add(Lists.<BakedQuad>newArrayList());
            }

            this.builderAmbientOcclusion = builderAmbientOcclusion;
            this.builderGui3d = builderGui3d;
            this.builderCameraTransforms = builderCameraTransforms;
        }

        public MultiTexturedBasedModel.Builder addFaceQuad(EnumFacing enumFacing, BakedQuad bakedQuad) {
            ((List) this.builderFaceQuads.get(enumFacing.ordinal())).add(bakedQuad);
            return this;
        }

        public MultiTexturedBasedModel.Builder addGeneralQuad(BakedQuad bakedQuad) {
            this.builderGeneralQuads.add(bakedQuad);
            return this;
        }

        public MultiTexturedBasedModel.Builder setTexture(HashMap<EnumFacing, TextureAtlasSprite> textureAtlasSprite) {
            this.builderTexture = textureAtlasSprite;
            return this;
        }

        public IBakedModel makeBakedModel() {
            if (this.builderTexture == null) {
                throw new RuntimeException("Missing particle!");
            } else {
                return new MultiTexturedBasedModel(this.builderGeneralQuads, this.builderFaceQuads, this.builderAmbientOcclusion, this.builderGui3d, this.builderTexture, this.builderCameraTransforms);
            }
        }
    }
}
